(function (window, App, $) {
    'use strict';
    var lpadChangeManagement = {
        //-------------------------------------------------------------------------- CA
        callServiceForQuickCreateCA: async function (securityContext, defaultSeverity = "Low") {
            try {
                return new Promise((resolve, reject) => {
                    let currTimestamp = new Date().getTime();
                    const dataForCA = {
                        version: 'v0',
                        changeaction: {
                            name: '',
                            revision: '-',
                            type: 'type_ChangeAction',
                            description: '',
                            attributes: [
                                {
                                    name: 'Estimated Completion Date',
                                    value: ''
                                },
                                {
                                    name: 'Severity',
                                    value: defaultSeverity
                                }
                            ]
                        }
                    };
                    const optionscreateCA = {
                        url: `${App.info.contextURL}resources/modeler/change/changeaction?tenant=OnPremise&timestamp=${currTimestamp}`,
                        contentType: App.ajax.contentType.JSON,
                        headers: { SecurityContext: `ctx::${securityContext}`, 'Content-Type': App.ajax.contentType.JSON },
                        method: App.ajax.requestType.POST,
                        data: JSON.stringify(dataForCA)
                    };

                    App.ajax.call(optionscreateCA)
                        .then(response => {

                            if (response.changeaction) {
                                resolve(response);
                            } else {
                                reject(response);
                            }
                        })
                        .catch(errorMessage => {
                            reject(errorMessage);
                            throw (errorMessage);
                        });
                });
            } catch (error) {
                App.log.error({ title: `callServiceForQuickCreateCA`, message: error }, {}, error);
            }
        },

        createOptionsForConnectToCA: function (securityContext, selectedIds, pidCA) {
            try {
                const proposedData = [];
                selectedIds.forEach(selectedId => {
                    let element = {
                        where: { id: `pid:${selectedId}` },
                        operation: { name: 'None' },
                        why: { comment: {} },
                        activities: [{ what: { operation: 'ChangeStatus', 'target state': 'Release' }, why: { comment: {} } }]
                    };
                    proposedData.push(element);
                });

                const dataForConnection = {
                    version: 'v0',
                    changeaction: {
                        cestamp: '0000',
                        add: {
                            proposed: proposedData
                        },
                        delete: {},
                        modify: {}
                    }
                };

                let currTimestamp = new Date().getTime();
                const options = {
                    url: `${App.info.contextURL}resources/modeler/change/changeaction/${pidCA}?proposed=1&tenant=OnPremise&timestamp=${currTimestamp}`,
                    contentType: App.ajax.contentType.JSON,
                    headers: { SecurityContext: `ctx::${securityContext}`, 'Content-Type': App.ajax.contentType.JSON },
                    method: 'PUT',
                    data: JSON.stringify(dataForConnection)
                };
                return options;
            } catch (error) {
                App.log.error({ title: `createOptionsForConnectToCA`, message: error }, {}, error);
            }
        },

        createChangeAction: async function (input) {
            try {
                const selectedIds = this.getSelectedObjectIds();
                this.widget.loader.start();

                let thisSecurityContext = await launchpad.lpadUtils.getSecurityContext();
                const newChangeActionData = await App.lpadChangeManagement.callServiceForQuickCreateCA(thisSecurityContext, input.options?.defaultSeverity);
                let nameNewCA = newChangeActionData.changeaction.name;
                App.toaster.success({ title: App.i18n.t("launchpad.message.newCAWasCreated"), message: nameNewCA });

                const optionForConnection = App.lpadChangeManagement.createOptionsForConnectToCA(thisSecurityContext, selectedIds, newChangeActionData.changeaction.id);
                const resultMessageData = await App.lpadChangeManagement.connectSelectedToChangeManagementObject(optionForConnection, nameNewCA, 'launchpad.message.errorCreatingCA');

                if (resultMessageData) {
                    App.lpadChangeManagement.announcementForAllObjects(resultMessageData.changeaction.proposed.length, nameNewCA);
                }

                this.widget.loader.stop();
                this.widget.refresh();


            } catch (e) {
                this.widget.loader.stop();
                if (e) {
                    App.log.error({ title: App.i18n.t("launchpad.message.errorCreatingCA"), message: launchpad.lpadUtils.getErrorMessage(e) }, {}, e);
                } else {
                    App.log.error(App.i18n.t("launchpad.message.errorCreatingCA"), {}, "");
                }

            };

        },

        addExistingChangeAction: async function (input) {
            try {
                this.loader.start();
                const selectedIds = App.page.context.relIds;
                let kindOfData = ['physicalid', 'name'];
                let dataExistingCA = await launchpad.lpadUtils.getInfoForObjectId(input.dataset.id, kindOfData, 'launchpad.message.errorGetCAInfo');
                let thisSecurityContext = await launchpad.lpadUtils.getSecurityContext();
                let physicalidConnectedCA = dataExistingCA[0];
                let nameConnectedCA = dataExistingCA[1];

                const optionForConnection = App.lpadChangeManagement.createOptionsForConnectToCA(thisSecurityContext, selectedIds, `pid:${physicalidConnectedCA}`);
                const resultMessageData = await App.lpadChangeManagement.connectSelectedToChangeManagementObject(optionForConnection, nameConnectedCA, 'launchpad.message.errorAddingToCA');

                App.lpadChangeManagement.announcementForOnlyNewObjects(resultMessageData, nameConnectedCA, selectedIds);

                this.loader.stop();
                this.close();
                App.page.context.widget.refresh();
            } catch (e) {
                this.loader.stop();
                if (e) {
                    App.log.error({ title: App.i18n.t("launchpad.message.errorAddingToCA"), message: launchpad.lpadUtils.getErrorMessage(e) }, {}, e);
                } else {
                    App.log.error(App.i18n.t("launchpad.message.errorAddingToCA"), {}, "");
                }
            };
        },

        //-------------------------------------------------------------------------- Common for Change management
        getInfoForNewChangeManagementObject: async function (securityContext, objectType, messageGetDataFailure) {
            try {
                return new Promise((resolve, reject) => {
                    let currTimestamp = new Date().getTime();

                    let urlGetObjectInfo = `${App.info.contextURL}resources/v1/collabServices/authoring/createContent/typeInfo?tenant=OnPremise&objType=${objectType}&preferedType=${objectType}&typeName=${objectType}&subTypes=true&timestamp${currTimestamp}=${currTimestamp}`;
                    const optionsGetObjectInfo = {
                        url: urlGetObjectInfo,
                        headers: { SecurityContext: parent.window.encodeURI(`ctx::${securityContext}`), 'Content-Type': App.ajax.contentType.JSON },
                        method: App.ajax.requestType.GET
                    };

                    App.ajax.call(optionsGetObjectInfo)
                        .then(response => {
                            if (response.status === 'success') {
                                resolve(response);
                            } else {
                                App.log.error(response.result[0].errors[0].errorCode, {}, response.result[0].errors[0].errorCode);
                                reject(response);
                            }
                        })
                        .catch(errorMessage => {
                            reject(errorMessage);
                            if (!errorMessage) {
                                let errMessage = App.i18n.t(`${messageGetDataFailure}`);
                                App.log.error(errMessage, {}, errMessage);
                            }
                            throw (errorMessage);
                        });
                });
            } catch (error) {
                App.log.error({ title: `getInfoForNewChangeManagementObject`, message: error }, {}, error);
            }
        },

        searchForAddExistingChangeManagementObject: function (input) {
            try {
                let data = this.selectedRows();
                let objectIds = [];
                for (var i = 0; i < data.length; i++) {
                    objectIds.push(data[i].objectId);
                }
                App.page.context.relIds = objectIds;
                App.page.context.widget = this.widget;
                App.searchV2.startSearch(input);
            } catch (error) {
                App.log.error({ title: `searchForAddExistingChangeManagementObject`, message: error }, {}, error);
            }
        },

        createNewChangeManagementObject: async function (securityContext, infoPayload, objectTypeData, messageCreationFailure, defaultAnalysisType = "Detailed", csrfValue) {
            try {
                return new Promise((resolve, reject) => {
                    const createObjectPayload = {
                        create: [{
                            attributes: infoPayload.result[0].attributes,
                            interfaces: infoPayload.result[0].interfaces,
                            type: infoPayload.result[0].type
                        }]
                    };

                    createObjectPayload.create[0].attributes.internalAttributes.forEach(internalAttr => {
                        if (internalAttr.name === 'name') {
                            for (let i = 0; i < createObjectPayload.create[0].attributes.publicAttributes.length; i++) {
                                if (createObjectPayload.create[0].attributes.publicAttributes[i].nls === 'Title') {
                                    createObjectPayload.create[0].attributes.publicAttributes[i].value = internalAttr.value;
                                    Object.assign(createObjectPayload.create[0].attributes.publicAttributes[i], { "modified": true });
                                    break;
                                }
                            }
                        }
                    });

                    if (objectTypeData === 'Change Request' && defaultAnalysisType === 'Simple') {
                        const infoCRType = createObjectPayload.create[0].attributes.publicAttributes.find(attrib => attrib.name === 'Analysis Type');
                        if (infoCRType) {
                            infoCRType.value = defaultAnalysisType;
                            Object.assign(infoCRType, { "modified": true });
                        }
                    }


                    let newHeaders = { SecurityContext: `ctx::${securityContext}`, 'Content-Type': App.ajax.contentType.JSON };
                    if (csrfValue) {
                        Object.assign(newHeaders, { ENO_CSRF_TOKEN: csrfValue });
                    }

                    let urlForCreateObject = `${App.info.contextURL}resources/v1/collabServices/authoring/createContent/Create?tenant=OnPremise&typeName=${objectTypeData}&parsed=false&activeFolder=false&`;
                    const optionsCreateObject = {
                        url: urlForCreateObject,
                        headers: newHeaders,
                        method: App.ajax.requestType.POST,
                        data: JSON.stringify(createObjectPayload)
                    };

                    App.ajax.call(optionsCreateObject)
                        .then(response => {
                            if (response.status === 'success') {
                                resolve(response);
                            } else {
                                App.log.error(response.result[0].errors[0].errorCode, {}, response.result[0].errors[0].errorCode);
                                reject(response);
                            }
                        })
                        .catch(errorMessage => {
                            reject(errorMessage);
                            if (!errorMessage) {
                                let errMessage = App.i18n.t(messageCreationFailure);
                                App.log.error(errMessage, {}, errMessage);
                            }
                            throw (errorMessage);
                        });

                });
            } catch (error) {
                App.log.error({ title: `createNewChangeManagementObject`, message: error }, {}, error);
            }

        },

        connectSelectedToChangeManagementObject: async function (payloadForConnection, objectName, messageConnectFailure) {
            try {
                return new Promise((resolve, reject) => {
                    App.ajax.call(payloadForConnection)
                        .then(response => {
                            resolve(response);
                        })
                        .catch(errorMessage => {
                            const errMessage = launchpad.lpadUtils.convertStringToObject(errorMessage);
                            if (!errMessage) {
                                let errText = `Error when attaching objects to ${objectName} !`;
                                App.log.error({ title: App.i18n.t(messageConnectFailure), message: errText }, {}, errText);
                            }
                            reject(errMessage);
                            throw (errMessage);
                        });
                });
            } catch (error) {
                App.log.error({ title: `connectSelectedToChangeManagementObject`, message: error }, {}, error);
            }
        },

        announcementForAllObjects: function (numberConnectedObjects, nameChangeManagementObject) {
            try {
                if (numberConnectedObjects === 1) {
                    App.toaster.success(`${App.i18n.t("launchpad.message.objectWasAttachedTo")} ${nameChangeManagementObject}`);
                } else {
                    App.toaster.success(`${numberConnectedObjects} ${App.i18n.t("launchpad.message.objectsWereAttachedTo")} ${nameChangeManagementObject}`);
                }
            } catch (error) {
                App.log.error({ title: `announcementForAllObjects`, message: error }, {}, error);
            }
        },

        announcementForOnlyNewObjects: function (resultMessageData, nameConnectedCA, selectedIds, nameSelectedCO) {
            try {
                let numberConnectedObjects = 0;
                if (resultMessageData) {
                    resultMessageData.changeaction.proposed.forEach(objectInfo => {
                        if (selectedIds.includes(objectInfo.where.id.substring(4))) {
                            numberConnectedObjects += 1;
                        }
                    })
                }
                let changeManagementObject = `${nameConnectedCA}`;
                if (nameSelectedCO) {
                    changeManagementObject = `${nameConnectedCA} <br> in ${nameSelectedCO}`;
                }
                App.lpadChangeManagement.announcementForAllObjects(numberConnectedObjects, changeManagementObject);
            } catch (error) {
                App.log.error({ title: `announcementForOnlyNewObjects`, message: error }, {}, error);
            }
        },
        //-------------------------------------------------------------------------- CR 
        createChangeRequest: async function (input) {
            try {

                this.widget.loader.start();
                const selectedIds = launchpad.lpadUtils.createArrayFromList(this.getSelectedObjectIds());
                let thisSecurityContext = await launchpad.lpadUtils.getSecurityContext();
                let thisCSRF = await launchpad.lpadUtils.getCSRFToken();

                const newChangeRequestPayload = await App.lpadChangeManagement.getInfoForNewChangeManagementObject(thisSecurityContext, 'Change Request', 'launchpad.message.errorGetCRInfo');
                const dataNewChangeRequest = await App.lpadChangeManagement.createNewChangeManagementObject(thisSecurityContext, newChangeRequestPayload,
                    'Change Request', 'launchpad.message.errorCreatingCR', input.options?.analysisType);
                let nameNewCR = newChangeRequestPayload.result[0].attributes.internalAttributes[0].value;
                let physicalIdNewCR = dataNewChangeRequest.result[0].identifier;
                App.toaster.success({ title: App.i18n.t("launchpad.message.newCRWasCreated"), message: nameNewCR });

                const payloadForConnection = App.lpadChangeManagement.createOptionsForConnectToContextObject(thisSecurityContext,
                    selectedIds, `changerequest/${physicalIdNewCR}/affectedItems`, thisCSRF.csrf.value, 'CONNECT');
                const resultMessageData = await App.lpadChangeManagement.connectSelectedToChangeManagementObject(payloadForConnection, nameNewCR, 'launchpad.message.errorCreatingCR');

                if (resultMessageData) {
                    App.lpadChangeManagement.announcementForAllObjects(resultMessageData.data.length, nameNewCR);
                }

                this.widget.loader.stop();
                this.widget.refresh();


            } catch (e) {
                this.widget.loader.stop();
                if (e) {
                    App.log.error({ title: App.i18n.t("launchpad.message.errorCreatingCR"), message: launchpad.lpadUtils.getErrorMessage(e) }, {}, e);
                } else {
                    App.log.error(App.i18n.t("launchpad.message.errorCreatingCR"), {}, "");
                }

            };

        },

        addExistingChangeRequest: async function (input) {
            try {
                this.loader.start();

                const selectedIds = launchpad.lpadUtils.createArrayFromList(App.page.context.relIds);
                let kindOfData = ['physicalid', 'name'];
                let dataExistingCR = await launchpad.lpadUtils.getInfoForObjectId(input.dataset.id, kindOfData, 'launchpad.message.errorGetCRInfo');
                let physicalidConnectedCR = dataExistingCR[0];
                let nameConnectedCR = dataExistingCR[1];

                let thisSecurityContext = await launchpad.lpadUtils.getSecurityContext();
                let thisCSRF = await launchpad.lpadUtils.getCSRFToken();

                const payloadForConnection = App.lpadChangeManagement.createOptionsForConnectToContextObject(thisSecurityContext, selectedIds,
                    `changerequest/${physicalidConnectedCR}/affectedItems`, thisCSRF.csrf.value, 'CONNECT');
                const resultMessageData = await App.lpadChangeManagement.connectSelectedToChangeManagementObject(payloadForConnection, nameConnectedCR, 'launchpad.message.errorAddingToCR');

                if (resultMessageData) {
                    App.lpadChangeManagement.announcementForAllObjects(resultMessageData.data.length, nameConnectedCR);
                }

                this.loader.stop();
                this.close();
                App.page.context.widget.refresh();
            } catch (e) {
                this.loader.stop();
                if (e) {
                    App.log.error({ title: App.i18n.t("launchpad.message.errorAddingToCR"), message: launchpad.lpadUtils.getErrorMessage(e) }, {}, e);
                } else {
                    App.log.error(App.i18n.t("launchpad.message.errorAddingToCR"), {}, "");
                }

            };

        },

        //-------------------------------------------------------------------------- CO 
        createChangeOrder: async function (input) {
            try {
                const selectedIds = this.getSelectedObjectIds();
                this.widget.loader.start();

                let thisSecurityContext = await launchpad.lpadUtils.getSecurityContext();

                const newChangeOrderPayload = await App.lpadChangeManagement.getInfoForNewChangeManagementObject(thisSecurityContext, 'Change Order', 'launchpad.message.errorGetCOInfo');
                let dataNewChangeOrder = await App.lpadChangeManagement.createNewChangeManagementObject(thisSecurityContext, newChangeOrderPayload, 'Change Order', 'launchpad.message.errorCreatingCO');
                let physicalidNewCO = dataNewChangeOrder.result[0].identifier;
                let nameNewCO = newChangeOrderPayload.result[0].attributes.internalAttributes[0].value;
                App.toaster.success({ title: App.i18n.t("launchpad.message.newCOWasCreated"), message: nameNewCO });

                const newChangeActionPayloadForCO = await App.lpadChangeManagement.getInfoForNewChangeManagementObject(thisSecurityContext, 'Change Action', 'launchpad.message.errorGetCAInfo');
                let dataNewChangeActionForCO = await App.lpadChangeManagement.createNewChangeManagementObject(thisSecurityContext, newChangeActionPayloadForCO, `Change Action&aggregating_context=${physicalidNewCO}`, 'launchpad.message.errorCreatingCA');
                let physicalidNewCA = dataNewChangeActionForCO.result[0].identifier;
                let nameNewCA = newChangeActionPayloadForCO.result[0].attributes.internalAttributes[0].value;
                App.toaster.success({ title: App.i18n.t("launchpad.message.newCAWasCreated"), message: nameNewCA });

                const optionForConnection = App.lpadChangeManagement.createOptionsForConnectToCA(thisSecurityContext, selectedIds, `pid:${physicalidNewCA}`);
                const resultMessageData = await App.lpadChangeManagement.connectSelectedToChangeManagementObject(optionForConnection, nameNewCA, 'launchpad.message.errorCreatingCA');

                if (resultMessageData) {
                    App.lpadChangeManagement.announcementForAllObjects(resultMessageData.changeaction.proposed.length, nameNewCA);
                }

                this.widget.loader.stop();
                this.widget.refresh();


            } catch (e) {
                this.widget.loader.stop();
                if (e) {
                    App.log.error({ title: App.i18n.t("launchpad.message.errorCreatingCO"), message: launchpad.lpadUtils.getErrorMessage(e) }, {}, e);
                } else {
                    App.log.error(App.i18n.t("launchpad.message.errorCreatingCO"), {}, "");
                }

            };
        },

        addExistingChangeOrder: async function (input) {
            try {
                this.loader.start();
                const selectedIds = App.page.context.relIds;
                let kindOfData = ['from[Change Action].to.physicalid', 'from[Change Action].to.current', 'from[Change Action].to.from[Proposed Activities]', 'from[Change Action].to.name', 'physicalid', 'name'];
                let dataCAInExistingCO = await launchpad.lpadUtils.getInfoForObjectId(input.dataset.id, kindOfData, 'launchpad.message.errorGetCAInfo');

                let physicalidConnectedCA = '';
                let nameConnectedCA = '';
                let physicalidSelectedCO = dataCAInExistingCO[4];
                let nameSelectedCO = dataCAInExistingCO[5];

                // get a suitable CA 
                if (dataCAInExistingCO[0]) {
                    const physicalidCA = dataCAInExistingCO[0].split('\x07');
                    const stateCA = dataCAInExistingCO[1].split('\x07');
                    const isConnectedObjectoToCA = dataCAInExistingCO[2].split('\x07');
                    const nameCA = dataCAInExistingCO[3].split('\x07');
                    for (let index = 0; index < stateCA.length; index++) {
                        if (stateCA[index] === 'Prepare' || stateCA[index] === 'In Work') {
                            physicalidConnectedCA = physicalidCA[index];
                            nameConnectedCA = nameCA[index];
                            if (isConnectedObjectoToCA[index] === 'TRUE') {
                                break;
                            }
                        }
                    }
                }

                // or create new CA
                let thisSecurityContext = await launchpad.lpadUtils.getSecurityContext();
                if (physicalidConnectedCA === App.Constants.String.EMPTY) {
                    let newChangeActionPayloadForCO = await App.lpadChangeManagement.getInfoForNewChangeManagementObject(thisSecurityContext, 'Change Action', 'launchpad.message.errorGetCAInfo');
                    let dataNewChangeActionForCO = await App.lpadChangeManagement.createNewChangeManagementObject(thisSecurityContext, newChangeActionPayloadForCO, `Change Action&aggregating_context=${physicalidSelectedCO}`, 'launchpad.message.errorCreatingCA');
                    physicalidConnectedCA = dataNewChangeActionForCO.result[0].identifier;
                    nameConnectedCA = newChangeActionPayloadForCO.result[0].attributes.internalAttributes[0].value;
                    App.toaster.success({ title: App.i18n.t("launchpad.message.newCAWasCreated"), message: newChangeActionPayloadForCO.result[0].attributes.internalAttributes[0].value });
                }
                // connect objects to CA
                const optionForConnection = App.lpadChangeManagement.createOptionsForConnectToCA(thisSecurityContext, selectedIds, `pid:${physicalidConnectedCA}`);
                const resultMessageData = await App.lpadChangeManagement.connectSelectedToChangeManagementObject(optionForConnection, nameConnectedCA, 'launchpad.message.errorAddingToCA');

                App.lpadChangeManagement.announcementForOnlyNewObjects(resultMessageData, nameConnectedCA, selectedIds, nameSelectedCO);

                this.loader.stop();
                this.close();
                App.page.context.widget.refresh();
            } catch (e) {
                this.loader.stop();
                if (e) {
                    App.log.error({ title: App.i18n.t("launchpad.message.errorAddingCO"), message: launchpad.lpadUtils.getErrorMessage(e) }, {}, e);
                } else {
                    App.log.error(App.i18n.t("launchpad.message.errorAddingCO"), {}, "");
                }

            };
        },
        //-------------------------------------------------------------------------- IA, AB & II
        callServiceForDeleteAI: async function (securityContext, dataDeleteAI) {
            try {
                return new Promise((resolve, reject) => {
                    let currTimestamp = new Date().getTime();
                    const optionsDeleteAI = {
                        url: `${App.info.contextURL}resources/lifecycle/Delete/structure?tenant=OnPremise&timestamp=${currTimestamp}`,
                        contentType: App.ajax.contentType.JSON,
                        headers: { SecurityContext: `ctx::${securityContext}`, 'Content-Type': App.ajax.contentType.JSON },
                        method: App.ajax.requestType.POST,
                        data: JSON.stringify(dataDeleteAI)
                    };

                    App.ajax.call(optionsDeleteAI)
                        .then(response => {
                            if (response.status === 'success') {
                                resolve(response);
                            } else {
                                reject(response);
                            }
                        })
                        .catch(errorMessage => {
                            const errMessage = launchpad.lpadUtils.convertStringToObject(errorMessage);
                            reject(errMessage);
                            throw (errMessage);
                        });
                });
            } catch (error) {
                App.log.error({ title: `callServiceForDeleteAI`, message: error }, {}, error);
            }
        },
        //  for CR "Change Request.Analysis Type" Detailed
        createImpactAnalysis: async function (input) {
            try {
                this.widget.loader.start();
                let physicalidContextCR = this.context.objectIds[0];
                let thisSecurityContext = await launchpad.lpadUtils.getSecurityContext();

                const csrfInfoAboutContextCR = await App.lpadChangeManagement.callServiceForGetChangeAnalysisObjectInfo(thisSecurityContext, `changerequest/${physicalidContextObject}`);
                let csrfContextCR = csrfInfoAboutContextCR.csrf.value;

                const newImpactAnalysisPayload = await App.lpadChangeManagement.getInfoForNewChangeManagementObject(thisSecurityContext, 'Change Analysis', 'launchpad.message.errorCreatingIA');
                let responseInfo = await App.lpadChangeManagement.createNewChangeManagementObject(thisSecurityContext, newImpactAnalysisPayload, `Change Analysis&aggregating_context=${physicalidContextCR}`, 'launchpad.message.errorCreatingIA', '', csrfContextCR);
                if (responseInfo.status === 'success') {
                    let nameNewIA = newImpactAnalysisPayload.result[0].attributes.internalAttributes[0].value;
                    App.toaster.success({ title: App.i18n.t("launchpad.message.newIAWasCreated"), message: nameNewIA });
                }

                this.widget.loader.stop();
                this.widget.refresh();

            } catch (e) {
                this.widget.loader.stop();
                if (e) {
                    App.log.error({ title: App.i18n.t("launchpad.message.errorCreatingIA"), message: launchpad.lpadUtils.getErrorMessage(e) }, {}, e);
                } else {
                    App.log.error(App.i18n.t("launchpad.message.errorCreatingIA"), {}, "");
                }

            };

        },

        //  for CR "Change Request.Analysis Type" Detailed
        deleteImpactAnalysis: async function (input) {
            try {
                this.widget.loader.start();

                let kindOfData = ['physicalid'];
                const dataElements = [];
                const idSelectedObjects = this.widget.selectedObjects;
                for (let index = 0; index < idSelectedObjects.length; index++) {
                    let dataIA = await launchpad.lpadUtils.getInfoForObjectId(idSelectedObjects[index], kindOfData, 'launchpad.message.errorGetIAInfo');
                    dataElements.push({ physicalid: dataIA[0] });
                }
                const dataPhysicalIdSelectedObjects = { data: dataElements, options: [] };

                let thisSecurityContext = await launchpad.lpadUtils.getSecurityContext();
                const responseInfo = await App.lpadChangeManagement.callServiceForDeleteAI(thisSecurityContext, dataPhysicalIdSelectedObjects);
                App.toaster.success(`${responseInfo.results.length} ${App.i18n.t("launchpad.message.selectedObjectsHasBeenDeleted")}`);

                this.widget.loader.stop();
                this.widget.refresh();

            } catch (e) {
                this.widget.loader.stop();
                if (e) {
                    App.log.error({ title: App.i18n.t("launchpad.message.errorDeletingIA"), message: launchpad.lpadUtils.getErrorMessage(e) }, {}, e);
                } else {
                    App.log.error(App.i18n.t("launchpad.message.errorDeletingIA"), {}, "");
                }

            };
        },

        createOptionsForConnectToContextObject: function (securityContext, selectedItems, idContextObject, csrfValue, updateAction) {
            try {
                const connectedData = { data: [] };
                selectedItems.forEach(selectedItem => {
                    let element = {
                        id: selectedItem[0],
                        updateAction: updateAction
                    };
                    if (selectedItem[1]) {
                        element.type = selectedItem[1];
                    }
                    connectedData.data.push(element);
                });

                let currTimestamp = new Date().getTime();
                const options = {
                    url: `${App.info.contextURL}resources/v1/chguxservices/${idContextObject}?tenant=OnPremise&timestamp=${currTimestamp}`,
                    contentType: App.ajax.contentType.JSON,
                    headers: { SecurityContext: `ctx::${securityContext}`, 'Content-Type': App.ajax.contentType.JSON, ENO_CSRF_TOKEN: csrfValue },
                    method: 'PUT',
                    data: JSON.stringify(connectedData)
                };
                return options;
            } catch (error) {
                App.log.error({ title: `createOptionsForConnectToContextObject`, message: error }, {}, error);
            }
        },

        callServiceForGetChangeAnalysisObjectInfo: async function (securityContext, physicalIdContextObject) {
            try {
                return new Promise((resolve, reject) => {
                    let currTimestamp = new Date().getTime();
                    const optionsGetCRInfo = {
                        url: `${App.info.contextURL}resources/v1/chguxservices/${physicalIdContextObject}/access?$simpleJSON=false&tenant=OnPremise&timestamp=${currTimestamp}`,
                        contentType: App.ajax.contentType.JSON,
                        headers: { SecurityContext: `ctx::${securityContext}`, 'Content-Type': App.ajax.contentType.JSON },
                        method: App.ajax.requestType.GET
                    };

                    App.ajax.call(optionsGetCRInfo)
                        .then(response => {
                            if (response.success) {
                                resolve(response);
                            } else {
                                reject(response);
                            }
                        })
                        .catch(errorMessage => {
                            const errMessage = launchpad.lpadUtils.convertStringToObject(errorMessage);
                            reject(errMessage);
                            throw (errMessage);
                        });
                });
            } catch (error) {
                App.log.error({ title: `callServiceForGetChangeAnalysisObjectInfo`, message: error }, {}, error);
            }
        },

        addChangeAnalysisItem: async function (input) {
            try {
                this.loader.start();

                let physicalidContextObject = input.contextOids[0];
                let kindOfData = ['physicalid'];
                if (this.dataOptions.itemType === 'impactedItems') {
                    kindOfData.push('type');
                }

                const idSearchedItems = input.selectedOids;
                const dataSearchedItems = [];
                for (let index = 0; index < idSearchedItems.length; index++) {
                    let dataItem = await launchpad.lpadUtils.getInfoForObjectId(idSearchedItems[index], kindOfData, 'launchpad.message.errorGetItemInfo');
                    dataSearchedItems.push(dataItem);
                }
                let nameContextObject = await launchpad.lpadUtils.getInfoForObjectId(physicalidContextObject, ['name'], 'launchpad.message.errorGetContextObjectInfo');
                let thisSecurityContext = await launchpad.lpadUtils.getSecurityContext();
                const infoAboutContextObject = await App.lpadChangeManagement.callServiceForGetChangeAnalysisObjectInfo(thisSecurityContext, `${this.dataOptions.parentType}/${physicalidContextObject}`);

                const payloadForConnection = App.lpadChangeManagement.createOptionsForConnectToContextObject(thisSecurityContext,
                    dataSearchedItems, `${this.dataOptions.parentType}/${physicalidContextObject}/${this.dataOptions.itemType}`,
                    infoAboutContextObject.csrf.value, 'CONNECT');

                const resultMessageData = await App.lpadChangeManagement.connectSelectedToChangeManagementObject(payloadForConnection, nameContextObject, 'launchpad.message.errorConnectingItems');

                if (resultMessageData) {
                    App.toaster.success(`${resultMessageData.data.length} ${App.i18n.t('launchpad.message.selectedObjectsHasBeenConnected')} ${nameContextObject[0]}`);
                }

                this.loader.stop();
                this.close();
                input.datatable.toolbar.widget.refresh()
            } catch (e) {
                this.loader.stop();
                if (e) {
                    App.log.error({ title: App.i18n.t("launchpad.message.errorConnectingItems"), message: launchpad.lpadUtils.getErrorMessage(e) }, {}, e);
                } else {
                    App.log.error(App.i18n.t("launchpad.message.errorConnectingItems"), {}, "");
                }
            };
        },

        detachChangeAnalysisItem: async function (input) {
            try {

                this.widget.loader.start();
                const idDetachedItems = this.getSelectedObjectIds();

                let physicalidContextObject = this.context.objectIds[0];
                let kindOfData = ['physicalid'];
                if (input.options.itemType === 'impactedItems') {
                    kindOfData.push('type');
                }

                const dataDetachedItems = [];
                for (let index = 0; index < idDetachedItems.length; index++) {
                    let dataItem = await launchpad.lpadUtils.getInfoForObjectId(idDetachedItems[index], kindOfData, 'launchpad.message.errorGetItemInfo');
                    dataDetachedItems.push(dataItem);
                }

                let nameContextObject = await launchpad.lpadUtils.getInfoForObjectId(physicalidContextObject, ['name'], 'launchpad.message.errorGetContextObjectInfo');
                let thisSecurityContext = await launchpad.lpadUtils.getSecurityContext();
                const infoAboutContextObject = await App.lpadChangeManagement.callServiceForGetChangeAnalysisObjectInfo(thisSecurityContext, `${input.options.parentType}/${physicalidContextObject}`);

                const payloadForDisconnection = App.lpadChangeManagement.createOptionsForConnectToContextObject(thisSecurityContext,
                    dataDetachedItems, `${input.options.parentType}/${physicalidContextObject}/${input.options.itemType}`,
                    infoAboutContextObject.csrf.value, input.options.connectionType);
                const resultMessageData = await App.lpadChangeManagement.connectSelectedToChangeManagementObject(payloadForDisconnection, nameContextObject, input.options.errorMessage);

                if (resultMessageData) {
                    App.toaster.success(`${idDetachedItems.length} ${App.i18n.t(input.options.successMessage)} ${nameContextObject[0]}`);
                }

                this.widget.loader.stop();
                if (input.options.connectionType === 'CONNECT') {
                    this.widget.dashboard.refresh();
                } else {
                    this.widget.refresh();
                }

            } catch (e) {
                this.widget.loader.stop();
                if (e) {
                    App.log.error({ title: App.i18n.t(input.options.errorMessage), message: launchpad.lpadUtils.getErrorMessage(e) }, {}, e);
                } else {
                    App.log.error(App.i18n.t(input.options.errorMessage), {}, "");
                }

            };

        }

    };
    App.lpadChangeManagement = $.extend({}, App.lpadChangeManagement || {}, lpadChangeManagement);
})(window, App, jQuery);


